<?php

    namespace Vepara\VeparaPos\Controller\Sanalpos;

    use Magento\Checkout\Model\Type\Onepage;
    use Magento\Framework\Controller\ResultFactory;
    use Magento\Framework\App\RequestInterface;
    use Magento\Framework\App\Request\InvalidRequestException;
    use Magento\Quote\Model\QuoteIdToMaskedQuoteIdInterface;

    class Response extends \Magento\Framework\App\Action\Action implements \Magento\Framework\App\CsrfAwareActionInterface
    {

        protected $_context;
        protected $_pageFactory;
        protected $_jsonEncoder;
        protected $_checkoutSession;
        protected $_customerSession;
        protected $_quote;
        protected $_cartManagement;
        protected $_resultRedirect;
        protected $_resultJsonFactory;
        protected $_quoteRepository;
        protected $_scopeConfig;
        protected $_request;
        protected $_messageManager;
        protected $_guestCartManagement;
        protected $_storeManager;
        protected $_quoteIdToMaskedQuoteId;


        public function __construct(
            \Magento\Framework\App\Action\Context                            $context,
            \Magento\Framework\Json\EncoderInterface                         $encoder,
            \Magento\Framework\View\Result\PageFactory                       $pageFactory,
            \Magento\Checkout\Model\Session                                  $checkoutSession,
            \Magento\Customer\Model\Session                                  $customerSession,
            \Magento\Quote\Model\Quote                                       $quote,
            \Magento\Quote\Api\CartManagementInterface                       $cartManagement,
            \Magento\Framework\Controller\ResultFactory                      $resultFactory,
            \Magento\Framework\Controller\Result\JsonFactory                 $resultJsonFactory,
            \Magento\Quote\Api\CartRepositoryInterface                       $quoteRepository,
            \Magento\Framework\App\Config\ScopeConfigInterface               $scopeConfig,
            \Magento\Framework\App\Request\Http                              $request,
            \Magento\Framework\Message\ManagerInterface                      $messageManager,
            \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $guestCartManagement,
            \Magento\Store\Model\StoreManagerInterface                       $storeManager,
            QuoteIdToMaskedQuoteIdInterface                                  $quoteIdToMaskedQuoteId
        )
        {
            parent::__construct($context);
            $this->_resultJsonFactory = $resultJsonFactory;
            $this->_pageFactory = $pageFactory;
            $this->_jsonEncoder = $encoder;
            $this->_checkoutSession = $checkoutSession;
            $this->_customerSession = $customerSession;
            $this->_quote = $quote;
            $this->_cartManagement = $cartManagement;
            $this->_resultRedirect = $context->getResultFactory();
            $this->_quoteRepository = $quoteRepository;
            $this->_scopeConfig = $scopeConfig;
            $this->_request = $request;
            $this->_messageManager = $messageManager;
            $this->_guestCartManagement = $guestCartManagement;
            $this->_storeManager = $storeManager;
            $this->_quoteIdToMaskedQuoteId = $quoteIdToMaskedQuoteId;

        }

        public function execute()
        {


            $sandbox = $this->_scopeConfig->getValue('payment/vepara_pos/sandbox');
            $api_key = $this->_scopeConfig->getValue('payment/vepara_pos/api_key');
            $api_secret = $this->_scopeConfig->getValue('payment/vepara_pos/api_secret');
            $resultRedirect = $this->_resultRedirect->create(ResultFactory::TYPE_REDIRECT);
            $checkoutSession = $this->_checkoutSession->getQuote();

            $payment_type = $checkoutSession->getPayment()->setMethod('vepara_pos');

            $this->_quote = $this->_checkoutSession->getQuote();

            $billingAddress = $checkoutSession->getBillingAddress();

            if ($billingAddress->getEmail()) {
                $email = $billingAddress->getEmail();
            } else {

                $email = $this->_customerSession->getCustomer()->getEmail();
            }


            if ($this->_customerSession->isLoggedIn()) {
                $customerId = $this->_customerSession->getCustomerId();
            }


            /*
             * 3d ödeme tamamlama bölümü
             */

            if (isset($_REQUEST['status_code']) && $_REQUEST['status_code'] == 100) {
                $sonuc = $this->validateHashKey($_REQUEST['hash_key'], $api_secret);
                if ($sonuc['0'] == $_REQUEST['payment_status'] && $sonuc['2'] == $_REQUEST['invoice_id'] && $sonuc['3'] == $_REQUEST['order_id']) {
                    if ($_REQUEST['transaction_type'] == "Auth") {
                        if ($this->_customerSession->isLoggedIn()) {

                            $this->_cartManagement->placeOrder($checkoutSession->getId());
                        } else {
                            // $quoteId = $this->getQuoteMaskId($checkoutSession->getId());
                            $quoteId = $this->getQuoteMaskId($_REQUEST['invoice_id']);

                            try {

                                $this->_guestCartManagement->savePaymentInformationAndPlaceOrder($quoteId, $email, $payment_type, $this->_checkoutSession->getBillingAddress());
                            } catch (\Exception $e) {
                                echo $e->getMessage();
                                exit;
                            }
                        }


                        $resultRedirect->setPath('checkout/onepage/success', ['_secure' => true]);
                        return $resultRedirect;

                    }
                }
            } elseif (isset($_REQUEST['error']) && !empty($_REQUEST['error'])) {
                $this->_messageManager->addErrorMessage($_REQUEST['error']);
                $resultRedirect->setPath('checkout/cart', ['_secure' => true]);
                return $resultRedirect;
            }

        }


        /**
         * @inheritDoc
         */
        public function createCsrfValidationException(
            RequestInterface $request
        ): ?InvalidRequestException
        {
            return null;
        }

        /**
         * @inheritDoc
         */
        public function validateForCsrf(RequestInterface $request): ?bool
        {
            return true;
        }

        private function getQuoteMaskId($quoteId)
        {
            $maskedId = null;
            try {
                $maskedId = $this->_quoteIdToMaskedQuoteId->execute($quoteId);
            } catch (NoSuchEntityException $exception) {
                throw new LocalizedException(__('Current user does not have an active cart.'));
            }

            return $maskedId;
        }


        function validateHashKey($hashKey, $secretKey)
        {
            $status = $currencyCode = "";
            $total = $invoiceId = $orderId = 0;
            if (!empty($hashKey)) {
                $hashkey = str_replace('__', '/', $hashKey);
                $password = sha1($secretKey);
                $components = explode(':', $hashkey);
                if (count($components) > 2) {
                    $iv = isset($components[0]) ? $components[0] : "";
                    $salt = isset($components[1]) ? $components[1] : "";
                    $salt = hash('sha256', $password . $salt);
                    $encryptedMsg = isset($components[2]) ? $components[2] : "";
                    $decryptedMsg = openssl_decrypt($encryptedMsg, 'aes-256-cbc', $salt, 0, $iv);
                    if (strpos($decryptedMsg, '|') !== false) {
                        $array = explode('|', $decryptedMsg);
                        $status = isset($array[0]) ? $array[0] : 0;
                        $total = isset($array[1]) ? $array[1] : 0;
                        $invoiceId = isset($array[2]) ? $array[2] : '0';
                        $orderId = isset($array[3]) ? $array[3] : 0;
                        $currencyCode = isset($array[4]) ? $array[4] : '';
                    }
                    return [$status, $total, $invoiceId, $orderId, $currencyCode];
                }
            }
        }


    }